import shutil
from datetime import datetime

from box import BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, convert_keys, zcc_param_map


class DevicesAPI(APIEndpoint):
    def download_devices(
        self,
        filename: str = None,
        os_types: list = None,
        registration_types: list = None,
    ):
        """
        Downloads the list of devices in the Client Connector Portal as a CSV file.

        By default, this method will create a file named `zcc-devices-YYmmDD-HH_MM_SS.csv`. This can be overridden by
        specifying the ``filename`` argument.

        Notes:
            This API endpoint is heavily rate-limited by Zscaler and as of NOV 2022 only 3 calls per-day are allowed.

        Args:
            filename (str):
                The name of the file that you want to save to disk.
            os_types (list):
                A list of OS Types to filter the device list. Omitting this argument will result in all OS types being
                matched.
                Valid options are:

                - ios
                - android
                - windows
                - macos
                - linux
            registration_types (list):
                A list of device registration states to filter the device list.
                Valid options are:

                - all (provides all states except for 'removed')
                - registered
                - removal_pending
                - unregistered
                - removed
                - quarantined

        Returns:
            :obj:`str`: The local filename for the CSV file that was downloaded.

        Examples:
            Create a CSV with all OS types and all registration types:

            >>> zcc.devices.download_devices(registration_types=["all", "removed"])

            Create a CSV for Windows and macOS devices that are in the `registered` state:

            >>> zcc.devices.download_devices(os_types=["windows", "macos"],
            ...     registration_types=["registered"])

            Create a CSV with filename `unregistered.csv` for devices in the unregistered state:

            >>> zcc.devices.download_devices(filename="unregistered.csv",
            ...     registration_types=["unregistered"])

        """

        if not filename:
            filename = f"zcc-devices-{datetime.now().strftime('%Y%m%d-%H_%M_%S')}.csv"

        payload = {
            "osTypes": [],
            "registrationTypes": [],
        }

        # Simplify the os_type argument, raise an error if the user supplies the wrong one.
        if os_types:
            for item in os_types:
                os_type = zcc_param_map["os"].get(item, None)
                if os_type:
                    payload["osTypes"].append(os_type)
                else:
                    raise ValueError("Invalid os_type specified. Check the pyZscaler documentation for valid os_type options.")

        # Simplify the registration_type argument, raise an error if the user supplies the wrong one.
        if registration_types:
            for item in registration_types:
                reg_type = zcc_param_map["reg_type"].get(item, None)
                if reg_type:
                    payload["registrationTypes"].append(reg_type)
                else:
                    raise ValueError(
                        "Invalid registration_type specified. Check the pyZscaler documentation for valid "
                        "registration_type options."
                    )

        # Create the local file and stream the device list csv to it
        with self._get("public/v1/downloadDevices", params=payload, stream=True) as r:
            with open(filename, "wb") as f:
                shutil.copyfileobj(r.raw, f)

        return filename

    def list_devices(self, **kwargs) -> BoxList:
        """
        Returns the list of devices enrolled in the Client Connector Portal.

        Keyword Args:
            os_type (str):
                Filter by device operating system. Valid options are:

                - ios
                - android
                - windows
                - macos
                - linux
            page (int):
                Return a specific page number.
            page_size (int):
                Specify the number of devices per page, defaults to ``30``.
            user_name (str):
                Filter by the enrolled user for the device.

        Returns:
            :obj:`BoxList`: A list containing devices using ZCC enrolled in the Client Connector Portal.

        Examples:
            Prints all devices in the Client Connector Portal to the console:

            >>> for device in zcc.devices.list_devices():
            ...    print(device)

        """
        payload = convert_keys(dict(kwargs))

        # Simplify the os_type argument, raise an error if the user supplies the wrong one.
        if kwargs.get("os_type"):
            os_type = zcc_param_map["os"].get(payload["osType"], None)
            if os_type:
                payload["osType"] = os_type
            else:
                raise ValueError("Invalid os_type specified. Check the pyZscaler documentation for valid os_type options.")

        return BoxList(Iterator(self._api, "public/v1/getDevices", **payload))

    def remove_devices(self, force: bool = False, **kwargs):
        """
        Removes the specified devices from the Zscaler Client Connector Portal.

        Notes:
            You must be using API credentials with the `Write` role.
            You must specify at least one criterion from `Keyword Args` to remove devices.

        Args:
            force (bool):
                Setting force to ``True`` removes the enrolled device from the portal. You can only remove devices that
                are in the `registered` or `device removal pending` state.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            client_connector_version (list):
                A list of client connector versions that will be removed. You must supply the exact version number, i.e.
                if the Client Connector version is `3.2.0.18` you must specify `3.2.0.18` and not `3.2`.
            os_type (str):
                The OS Type for the devices to be removed. Valid options are:

                - ios
                - android
                - windows
                - macos
                - linux
            udids (list):
                A list of Unique Device IDs.
            user_name (str):
                The username of the user whose devices will be removed.

        Returns:
            :obj:`Box`: Server response containing the total number of devices removed.

        Examples:
            Soft-remove devices using ZCC version 3.7.1.44 from the Client Connector Portal:

            >>> zcc.devices.remove_devices(client_connector_version=["3.7.1.44"])

            Soft-remove Android devices from the Client Connector Portal:

            >>> zcc.devices.remove_devices(os_type="android")

            Hard-remove devices from the Client Connector Portal by UDID:

            >>> zcc.devices.remove_devices(force=True, udids=["99999", "88888", "77777"])

            Hard-remove Android devices for johnno@widgets.co from the Client Connector Portal:

            >>> zcc.devices.remove_devices(force=True, os_type="android",
            ...     user_name="johnno@widgets.co")

        """
        payload = convert_keys(dict(kwargs))

        # Simplify the os_type argument, raise an error if the user supplies the wrong one.
        if kwargs.get("os_type"):
            os_type = zcc_param_map["os"].get(payload["osType"], None)
            if os_type:
                payload["osType"] = os_type
            else:
                raise ValueError("Invalid os_type specified. Check the pyZscaler documentation for valid os_type options.")

        if force:
            return self._post("public/v1/forceRemoveDevices", json=payload)
        else:
            return self._post("public/v1/removeDevices", json=payload)



from box import Box
from restfly import APIEndpoint

from pyzscaler.utils import obfuscate_api_key


class ZCONSessionAPI(APIEndpoint):
    def status(self) -> Box:
        """
        Returns the status of the authentication session if it exists.

        Returns:
            :obj:`Box`: Session authentication information.

        Examples:
            Check the status of the authentication session::

                print(zcon.session.status())
        """
        return self._get("auth")

    def create(self, api_key: str, username: str, password: str) -> Box:
        """
        Create a new ZCON authentication session.

        Args:
            api_key (str): The ZCON API Key.
            username (str): Username of admin user for the authentication session.
            password (str): Password of the admin user for the authentication session.

        Returns:
            :obj:`Box`: The authenticated session information.

        Examples:
            Create a new authentication session::

                zcon.session.create(
                    api_key='123456789',
                    username='admin@example.com',
                    password='MyInsecurePassword'
                )
        """
        api_obf = obfuscate_api_key(api_key)

        payload = {
            "apiKey": api_obf["key"],
            "username": username,
            "password": password,
            "timestamp": api_obf["timestamp"],
        }
        return self._post("auth", json=payload)

    def delete(self) -> int:
        """
        End the authentication session.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            End the authentication session::

                print(zcon.session.delete())
        """
        return self._delete("auth").status



from box import Box, BoxList
from restfly import APISession
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator


class TrustedNetworksAPI(APIEndpoint):
    def __init__(self, api: APISession):
        super().__init__(api)

        self.v2_url = api.v2_url

    def list_networks(self, **kwargs) -> BoxList:
        """
        Returns a list of all configured trusted networks.

        Keyword Args:
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: A list of all configured trusted networks.

        Examples:
            >>> for trusted_network in zpa.trusted_networks.list_networks():
            ...    pprint(trusted_network)

        """
        return BoxList(Iterator(self._api, f"{self.v2_url}/network", **kwargs))

    def get_network(self, network_id: str) -> Box:
        """
        Returns information on the specified trusted network.

        Args:
            network_id (str):
                The unique identifier for the trusted network.

        Returns:
            :obj:`Box`: The resource record for the trusted network.

        Examples:
            >>> pprint(zpa.trusted_networks.get_network('99999'))

        """

        return self._get(f"network/{network_id}")



from box import Box
from restfly.endpoint import APIEndpoint


class AppTotalAPI(APIEndpoint):
    def get_app(self, app_id: str, verbose: bool = False) -> Box:
        """
        Searches the AppTotal App Catalog by app ID. If the app exists in the catalog, the app's information is
        returned. If not, the app is submitted for analysis. After analysis is complete, a subsequent GET request is
        required to fetch the app's information.

        Args:
            app_id (str): The app ID to search for.
            verbose (bool, optional): Defaults to False.

        Returns:
            :obj:`Box`: The response object.

        Examples:
            Return verbose information on an app with ID 12345::

                zia.apptotal.get_app(app_id="12345", verbose=True)

        """
        params = {
            "app_id": app_id,
            "verbose": verbose,
        }
        return self._get("apps/app", params=params)

    def scan_app(self, app_id: str) -> Box:
        """
        Submits an app for analysis in the AppTotal Sandbox. After analysis is complete, a subsequent GET request is
        required to fetch the app's information.

        Args:
            app_id (str): The app ID to scan.

        Returns:
            :obj:`Box`: The response object.

        Examples:
            Scan an app with ID 12345::

                zia.apptotal.scan_app(app_id="12345")

        """
        payload = {
            "appId": app_id,
        }
        return self._post("apps/app", json=payload)



from box import BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import ZDXIterator, zdx_params


class AppsAPI(APIEndpoint):
    @zdx_params
    def list_apps(self, **kwargs) -> BoxList:
        """
        Returns a list of all active applications configured within the ZDX tenant.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`BoxList`: The list of applications in ZDX.

        Examples:
            List all applications in ZDX for the past 2 hours:

            >>> for app in zdx.apps.list_apps():
            ...     print(app)

        """
        return self._get("apps", params=kwargs)

    @zdx_params
    def get_app(self, app_id: str, **kwargs):
        """
        Returns information on the specified application configured within the ZDX tenant.

        Args:
            app_id (str): The unique ID for the ZDX application.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`Box`: The application information.

        Examples:
            Return information on the application with the ID of 999999999:

            >>> zia.apps.get_app(app_id='999999999')

        """
        return self._get(f"apps/{app_id}", params=kwargs)

    @zdx_params
    def get_app_score(self, app_id: str, **kwargs):
        """
        Returns the ZDX score trend for the specified application configured within the ZDX tenant.

        Args:
            app_id (str): The unique ID for the ZDX application.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`Box`: The application's ZDX score trend.

        Examples:
            Return the ZDX score trend for the application with the ID of 999999999:

            >>> zia.apps.get_app_score(app_id='999999999')

        """
        return self._get(f"apps/{app_id}/score", params=kwargs)

    @zdx_params
    def get_app_metrics(self, app_id: str, **kwargs):
        """
        Returns the ZDX metrics for the specified application configured within the ZDX tenant.

        Args:
            app_id (str): The unique ID for the ZDX application.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.
            metric_name (str): The name of the metric to return. Available values are:
                * `pft` - Page Fetch Time
                * `dns` - DNS Time
                * `availability`

        Returns:
            :obj:`Box`: The application's ZDX metrics.

        Examples:
            Return the ZDX metrics for the application with the ID of 999999999:

            >>> zia.apps.get_app_metrics(app_id='999999999')

            Return the ZDX metrics for the app with an ID of 999999999 for the last 24 hours, including dns matrics,
            geolocation, department and location IDs:

            >>> zia.apps.get_app_metrics(app_id='999999999', since=24, metric_name='dns', location_id='888888888',
            ...                          geo_id='777777777', department_id='666666666')

        """
        return self._get(f"apps/{app_id}/metrics", params=kwargs)

    @zdx_params
    def list_app_users(self, app_id: str, **kwargs):
        """
        Returns a list of users and devices that were used to access the specified application configured within
        the ZDX tenant.

        Args:
            app_id (str): The unique ID for the ZDX application.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.
            score_bucket (str): The ZDX score bucket to filter by. Available values are:
                * `poor` - 0-33
                * `okay` - 34-65
                * `good` - 66-100

        Returns:
            :obj:`BoxList`: The list of users and devices used to access the application.

        Examples:
            Return a list of users and devices who have accessed the application with the ID of 999999999:

            >>> for user in zia.apps.get_app_users(app_id='999999999'):
            ...     print(user)

        """
        return BoxList(
            ZDXIterator(
                self._api,
                f"apps/{app_id}/users",
                pagination="offset_limit",
                **kwargs,
            )
        )

    @zdx_params
    def get_app_user(self, app_id: str, user_id: str, **kwargs):
        """
        Returns information on the specified user and device that was used to access the specified application
        configured within the ZDX tenant.

        Args:
            app_id (str): The unique ID for the ZDX application.
            user_id (str): The unique ID for the ZDX user.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The user and device information.

        Examples:
            Return information on the user with the ID of 999999999 who has accessed the application with the ID of
            888888888:

            >>> zia.apps.get_app_user(app_id='888888888', user_id='999999999')

        """
        return self._get(f"apps/{app_id}/users/{user_id}", params=kwargs)



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator


class MachineGroupsAPI(APIEndpoint):
    def list_groups(self, **kwargs) -> BoxList:
        """
        Returns a list of all configured machine groups.

        Keyword Args:
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`list`: A list of all configured machine groups.

        Examples:
            >>> for machine_group in zpa.machine_groups.list_groups():
            ...    pprint(machine_group)

        """
        return BoxList(Iterator(self._api, "machineGroup", **kwargs))

    def get_group(self, group_id: str) -> Box:
        """
        Returns information on the specified machine group.

        Args:
            group_id (str):
                The unique identifier for the machine group.

        Returns:
            :obj:`Box`: The resource record for the machine group.

        Examples:
            >>> pprint(zpa.machine_groups.get_group('99999'))

        """

        return self._get(f"machineGroup/{group_id}")
